#!/usr/bin/perl -w
#

# 
# Parse cgminer JSON stats for use in Cacti
#
######################
##       Usage      ##
######################
#
# Summary
# 	query_cgminer_stats.pl HOST summary
#
# Pools
# 	query_cgminer_stats.pl HOST pools index
#
# Devices
# 	query_cgminer_stats.pl HOST devs index
#
######################
## Revision History ##
######################
# 
# v0.01 4/15/2012 JinTu (jintuNO@SPAMpraecogito.com)
#	First working version. Supports cgminer >=2.1.0 API JSON format (>=2.3.2 for PGA support). 
#	Supports stats from summary, pools and devs responses. Only tested with GPU support as no 
#       PGA or CPU miners were available at the time of release (caveat emptor).
#
##

use strict;
use IO::Socket;
use JSON -support_by_pp;

my $cgminer_host = $ARGV[0];
my $cgminer_port = "4028";
my $command = $ARGV[1];

unless ((defined $cgminer_host) && (($ARGV[1] eq "summary") || ($ARGV[1] eq "pools") || ($ARGV[1] eq "devs"))) {
  # Missing or malformatted input params
  usage();
}

my $remote = IO::Socket::INET->new(
			Proto		=> "tcp",
			PeerAddr	=> $cgminer_host,
			PeerPort	=> $cgminer_port,
                                  ) or die "cannot connect to port $cgminer_host:$cgminer_port, $!\n";

# Send the command to CGMINER
print $remote '{"command":"' . $command . '"}';


# Now read the result
my $response = <$remote>;

#print $command . " returned \'$response\'\n"; 

# Now parse the JSON
my $json = new JSON;

# probably want to add some error checking here

# these are some nice json options to relax restrictions a bit:
my $json_text = $json->allow_nonref->utf8->relaxed->escape_slash->loose->allow_singlequote->allow_barekey->decode($response);

if ($command eq "summary") {
  my $outstring = "";
  foreach my $cat (keys %{$json_text->{"SUMMARY"}->[0]}) {
    my $clean_cat = strip_spaces($cat);
    my $val = $json_text->{"SUMMARY"}->[0]->{$cat};
    if ($cat eq "Total MH") {
      $val = shiftdec($val,4);
    }
    elsif (($cat eq "MHS av") || ($cat eq "Utility")) {
      $val = shiftdec($val,2);
    }
    $outstring .= "$clean_cat:$val ";
  }
  $outstring =~ s/\s$//; # trim the trailing space
  print $outstring;
} 
else {
  unless (defined $ARGV[2]) {
    usage();
  }
  if ($json_text->{uc($command)} =~ /ARRAY/) {
    if ($ARGV[2] eq "num_indexes") {
      print scalar(@{$json_text->{uc($command)}}); 
    }
    elsif ($command eq "devs") {
      my %gpu;
      my %cpu;
      my %pga;
      # Define the indexes for the devices
      for (my $i = 0; $i <= (scalar @{$json_text->{uc($command)}} - 1); $i++) {
        if (defined $json_text->{uc($command)}->[$i]->{"CPU"}) {
          $cpu{"CPU" . $json_text->{uc($command)}->[$i]->{"CPU"}} = $i;
        }
        elsif (defined $json_text->{uc($command)}->[$i]->{"GPU"}) {
          $gpu{"GPU" . $json_text->{uc($command)}->[$i]->{"GPU"}} = $i;
        }
        elsif (defined $json_text->{uc($command)}->[$i]->{"PGA"}) {
          $pga{"PGA" . $json_text->{uc($command)}->[$i]->{"PGA"}} = $i;
        }
      }
      
      if ($ARGV[2] eq "index") {
        if (%cpu) {
          foreach my $dev (keys %cpu) {
            print "$dev\n";
          }
        }
        if (%gpu) {
          foreach my $dev (keys %gpu) {
            print "$dev\n";
          }
        }
        if (%pga) {
          foreach my $dev (keys %pga) {
            print "$dev\n";
          }
        }
      }
      elsif (($ARGV[2] eq "query") && (defined $ARGV[3])) {
        if ($ARGV[3] eq "device_name") {
          if (%cpu) {
            foreach my $dev (keys %cpu) {
              print "$dev:$dev\n";
            }
          }
          if (%gpu) {
            foreach my $dev (keys %gpu) {
              print "$dev:$dev\n";
            }
          }
          if (%pga) {
            foreach my $dev (keys %pga) {
              print "$dev:$dev\n";
            }
          }
        }
        else { 
          if (%cpu) {
            foreach my $dev (keys %cpu) {
              if (defined $json_text->{uc($command)}->[$cpu{$dev}]->{add_spaces($ARGV[3])}) {
                if ($ARGV[3] eq "Total_MH") {
                  print "$dev:" . shiftdec($json_text->{uc($command)}->[$cpu{$dev}]->{add_spaces($ARGV[3])}, 4) . "\n";
                }
                elsif (($ARGV[3] eq "Utility") || ($ARGV[3] eq "MHS_5s") || ($ARGV[3] eq "MHS_av") || ($ARGV[3] eq "Temperature")) {
                  print "$dev:" . shiftdec($json_text->{uc($command)}->[$cpu{$dev}]->{add_spaces($ARGV[3])}, 2) . "\n";
                }
                else {
                  print "$dev:" . $json_text->{uc($command)}->[$cpu{$dev}]->{add_spaces($ARGV[3])} . "\n";
                }
              }
            }
          }
          if (%gpu) {
            foreach my $dev (keys %gpu) {
              if (defined $json_text->{uc($command)}->[$gpu{$dev}]->{add_spaces($ARGV[3])}) {
                if ($ARGV[3] eq "Total_MH") {
                  print "$dev:" . shiftdec($json_text->{uc($command)}->[$gpu{$dev}]->{add_spaces($ARGV[3])}, 4) . "\n";
                }
                elsif (($ARGV[3] eq "Utility") || ($ARGV[3] eq "MHS_5s") || ($ARGV[3] eq "MHS_av") || ($ARGV[3] eq "Temperature")) {
                  print "$dev:" . shiftdec($json_text->{uc($command)}->[$gpu{$dev}]->{add_spaces($ARGV[3])}, 2) . "\n";
                }
                elsif ($ARGV[3] eq "GPU_Voltage") {
                  print "$dev:" . shiftdec($json_text->{uc($command)}->[$gpu{$dev}]->{add_spaces($ARGV[3])}, 3) . "\n";
                }
                else {
                  print "$dev:" . $json_text->{uc($command)}->[$gpu{$dev}]->{add_spaces($ARGV[3])} . "\n";
                }
              }
            }
          }
          if (%pga) {
            foreach my $dev (keys %pga) {
              if (defined $json_text->{uc($command)}->[$pga{$dev}]->{add_spaces($ARGV[3])}) {
                if ($ARGV[3] eq "Total_MH") {
                  print "$dev:" . shiftdec($json_text->{uc($command)}->[$pga{$dev}]->{add_spaces($ARGV[3])}, 4) . "\n";
                }
                elsif (($ARGV[3] eq "Utility") || ($ARGV[3] eq "MHS_5s") || ($ARGV[3] eq "MHS_av") || ($ARGV[3] eq "Temperature")) {
                  print "$dev:" . shiftdec($json_text->{uc($command)}->[$pga{$dev}]->{add_spaces($ARGV[3])}, 2) . "\n";
                }
                else {
                  print "$dev:" . $json_text->{uc($command)}->[$pga{$dev}]->{add_spaces($ARGV[3])} . "\n";
                }
              }
            }
          }
        }
      }
      elsif (($ARGV[2] eq "get") && (defined $ARGV[3])) {
        if ($ARGV[3] eq "device_name") {
          # just assume they got it right 
          print $ARGV[4];
        }
        elsif ($ARGV[4] =~ /CPU/) {
          if (defined $json_text->{uc($command)}->[$cpu{$ARGV[4]}]->{add_spaces($ARGV[3])}) {
            if ($ARGV[3] eq "Total_MH") {
              print shiftdec($json_text->{uc($command)}->[$cpu{$ARGV[4]}]->{add_spaces($ARGV[3])}, 4);
            }
            elsif (($ARGV[3] eq "Utility") || ($ARGV[3] eq "MHS_5s") || ($ARGV[3] eq "MHS_av") || ($ARGV[3] eq "Temperature")) {
              print shiftdec($json_text->{uc($command)}->[$cpu{$ARGV[4]}]->{add_spaces($ARGV[3])}, 2);
            }
            else {
              print $json_text->{uc($command)}->[$cpu{$ARGV[4]}]->{add_spaces($ARGV[3])};
            }
          }
        }
        elsif ($ARGV[4] =~ /GPU/) {
          if (defined $json_text->{uc($command)}->[$gpu{$ARGV[4]}]->{add_spaces($ARGV[3])}) {
            if ($ARGV[3] eq "Total_MH") {
              print shiftdec($json_text->{uc($command)}->[$gpu{$ARGV[4]}]->{add_spaces($ARGV[3])}, 4);
            }
            elsif (($ARGV[3] eq "Utility") || ($ARGV[3] eq "MHS_5s") || ($ARGV[3] eq "MHS_av") || ($ARGV[3] eq "Temperature")) {
              print shiftdec($json_text->{uc($command)}->[$gpu{$ARGV[4]}]->{add_spaces($ARGV[3])}, 2);
            }
            elsif ($ARGV[3] eq "GPU_Voltage") {
              print shiftdec($json_text->{uc($command)}->[$gpu{$ARGV[4]}]->{add_spaces($ARGV[3])}, 3);
            }
            else {
              print $json_text->{uc($command)}->[$gpu{$ARGV[4]}]->{add_spaces($ARGV[3])};
            }
          }
        }
        elsif ($ARGV[4] =~ /PGA/) {
          if (defined $json_text->{uc($command)}->[$pga{$ARGV[4]}]->{add_spaces($ARGV[3])}) {
            print $json_text->{uc($command)}->[$pga{$ARGV[4]}]->{add_spaces($ARGV[3])};
            if ($ARGV[3] eq "Total_MH") {
              print shiftdec($json_text->{uc($command)}->[$pga{$ARGV[4]}]->{add_spaces($ARGV[3])}, 4);
            }
            elsif (($ARGV[3] eq "Utility") || ($ARGV[3] eq "MHS_5s") || ($ARGV[3] eq "MHS_av") || ($ARGV[3] eq "Temperature")) {
              print shiftdec($json_text->{uc($command)}->[$pga{$ARGV[4]}]->{add_spaces($ARGV[3])}, 2);
            }
            else {
              print $json_text->{uc($command)}->[$pga{$ARGV[4]}]->{add_spaces($ARGV[3])};
            }
          }
        }
      }

    }
    elsif ($command eq "pools") {
      my %pools;
      for (my $i = 0; $i <= (scalar @{$json_text->{uc($command)}} - 1); $i++) {
        $pools{strip_http($json_text->{uc($command)}->[$i]->{"URL"})} = $i;
      }
      if ($ARGV[2] eq "index") {
        foreach my $pool (keys %pools) {
          print "$pool\n";
        }
      }
      elsif ($ARGV[2] eq "num_indexes") {
        print scalar(@{$json_text->{uc($command)}}); 
      }
      elsif (($ARGV[2] eq "query") && (defined $ARGV[3])) {
        foreach my $pool (keys %pools) {
          if ($ARGV[3] eq "URL") { 
            print "$pool:" . strip_http($json_text->{uc($command)}->[$pools{$pool}]->{add_spaces($ARGV[3])}) . "\n";
          }
          else {
            print "$pool:" . $json_text->{uc($command)}->[$pools{$pool}]->{add_spaces($ARGV[3])} . "\n";
          }
        }
      }
      elsif (($ARGV[2] eq "get") && (defined $ARGV[4])) {
        if ($ARGV[3] eq "URL") {
            print strip_http($json_text->{uc($command)}->[$pools{$ARGV[4]}]->{add_spaces($ARGV[3])}) . "\n";
          }
          else {
            print $json_text->{uc($command)}->[$pools{$ARGV[4]}]->{add_spaces($ARGV[3])} . "\n";
          }
      }
    }
  }
}

# make sure we close the socket
$remote->close();


# Subs from here on

# Shift the decimal place to the right by some number of digits
sub shiftdec {
  my ($val,$digits) = @_;
  return $val * (10 ** $digits);
}

# Strip spaces and replace with _
sub strip_spaces {
  my ($thestr) = @_;
  $thestr =~ s/ /_/g;
  return $thestr;
}

# Strip the http(s):// and replace : with _
sub strip_http {
  my ($thestr) = @_;
  $thestr =~ s/https:\/\///g;
  $thestr =~ s/http:\/\///g;
  $thestr =~ s/:/_/g;
  return $thestr;
}

# Strip the _ and add the spaces back in
sub add_spaces {
  my ($thestr) = @_;
  $thestr =~ s/_/ /g;
  return $thestr;
}

# Print usage info
sub usage {
  # Missing or malformatted input params
  print "usage:\n\n./query_cgminer_stats.pl HOST summary\n\n" .
	"./query_cgminer_stats.pl HOST pools index\n" .
	"./query_cgminer_stats.pl HOST pools num_indexes\n" .
	"./query_cgminer_stats.pl HOST pools query {URL,POOL,LongPoll,Stale,Status,Discarded,Get_Failures,Priority,Getworks,Remote_Failures,Accepted,Rejected}\n" .
	"./query_cgminer_stats.pl HOST pools get {URL,POOL,LongPoll,Stale,Status,Discarded,Get_Failures,Priority,Getworks,Remote_Failures,Accepted,Rejected} POOL\n\n" .
	"./query_cgminer_stats.pl HOST devs index\n" .
	"./query_cgminer_stats.pl HOST devs num_indexes\n" .
	"./query_cgminer_stats.pl HOST devs query {Total_MH,Last_Share_Time,Fan_Percent,Fan_Speed,Status,Last_Share_Pool,Utility,MHS_5s,Temperature,Memory_Clock,GPU_Activity,Accepted,Hardware_Errors,MHS_av,Enabled,GPU_Voltage,Rejected,GPU_Clock,GPU,Intensity,Powertune,CPU,PGA,Name,ID}\n" .
	"./query_cgminer_stats.pl HOST devs get {Total_MH,Last_Share_Time,Fan_Percent,Fan_Speed,Status,Last_Share_Pool,Utility,MHS_5s,Temperature,Memory_Clock,GPU_Activity,Accepted,Hardware_Errors,MHS_av,Enabled,GPU_Voltage,Rejected,GPU_Clock,GPU,Intensity,Powertune,CPU,PGA,Name,ID} DEV\n";
  exit 0;
}
